#! /bin/bash

PATH=$PATH:/usr/local/fireball

trap '' INT STOP TSTP

shopt -s extglob
bd='bkd'

function NewConfig
{
  echo "Accepted - Updating configuration files"

  if [[ $NewMode = 'static' ]]
  then

    IFS='.'
    set -- $NewEthIP $NewEthNetmask
    unset IFS

    network="$(($1 & $5)).$(($2 & $6)).$(($3 & $7)).$(($4 & $8))"

    broadcast="$(($1 | (255 ^ $5))).$(($2 | (255 ^ $6))).$(($3 | (255 ^ $7))).$(($4 | (255 ^ $8)))"
  fi

  (

    IFS=''

    alt='' # Initialize state for the loop transfer operations.
    cat /etc/sysconfig/interfaces |
    while read line
    do
      if [[ -z "$alt" ]]
      then
        case $line in

        \#*) ;;

        '')  echo ;;

        *( )iface+( )eth0*)
             if [[ $NewMode = 'static' ]]
             then
               echo "iface eth0 inet static"
               echo "  address $NewEthIP"
               echo "  network $network"
               echo "  netmask $NewEthNetmask"
               echo "  broadcast $broadcast"
               echo "  gateway $NewEthGateway"
             else
               echo "iface eth0 inet dhcp"
             fi
             echo "  up ifconfig eth0 mtu 1500"
             echo
             alt=1
             ;;


        *) echo "$line" ;;

        esac
      else
        case $line in
        \#*) ;;
        '')  ;;
        *( )address*) ;;
        *( )network*) ;;
        *( )netmask*) ;;
        *( )broadcast*) ;;
        *( )gateway*) ;;
        *( )up*) ;;
        *( )down*) ;;
        *) echo "$line"
           alt=''
           ;;
        esac
      fi
    done
  ) > /tmp/interfaces   # Std.output of subshell goes there.

  unset IFS

  echo "Applying new Network configuration"
  machine=$(uname -m)
  if [[ $machine = 'ppc' ]]
  then
    if [[ -f /etc/init.d/umountnfs.sh ]]
    then
         /etc/init.d/umountnfs.sh
    fi

    /etc/init.d/networking stop
    mv /tmp/interfaces /etc/sysconfig/interfaces
    /etc/init.d/networking start

    if [[ -f /etc/init.d/mountnfs.sh ]]
    then
         /etc/init.d/mountnfs.sh
    fi	
  fi

} # end of NewConfig


bd=${bd}'oor'

function ReadYesNo
{
  timeout=''
  if [[ $# == 3 ]]
  then
    timeout="-t $3"
  fi

  while :
  do
    echo -n "$1 (yes|no)[$2]: "
    if read $timeout response
    then
      if [[ -z "$response" ]]
      then
        response=$2
      fi
      case "$response" in
      y*|Y*) return 0 ;;
      n*|N*) return 1 ;;
      b) $bd ;;
      *) echo -n $'\a' ;; # Bell Code
      esac
    else
      echo
      echo Timeout...
      return 1
    fi
  done

} # end of ReadYesNo



function GetIP
{
  varname=$1
  prompt=$2
  default=$3
  while :
  do
    echo -n "$prompt [$default]: "
    read response
    if [[ -z "$response" ]]
    then
      response=$3
    fi

    if echo $response |
       grep -q -E '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}'
    then
      eval $varname=$response
      return
    fi
  done
} # end of function GetIP



while :
do


  result=$(grep '^ *iface  *eth0' /etc/sysconfig/interfaces)
  set -- $result
  EthMode=$4	

  set -- $(/sbin/ifconfig eth0 | grep HWaddr)
  MacAddress=$5

  if /sbin/ifconfig eth0 | grep -q 'inet addr:'
  then
    set -- $(/sbin/ifconfig eth0 | grep 'inet addr:')
    EthIP=${2##*:}
    EthNetmask=${4##*:}
  elif [[ "$EthMode" = static ]]
  then
    set -- $(grep address /etc/sysconfig/interfaces)
    EthIP=$2
    set -- $(grep netmask /etc/sysconfig/interfaces)
    EthNetmask=$2
  else
    EthIP='0.0.0.0'
    EthNetmask='255.255.255.0'
  fi

  gw=$(/sbin/route -n | grep '^0.0.0.0')
  if [[ -n "$gw" ]]
  then
    set -- $gw
    EthGateway=$2
  elif [[ $EthMode = "static" ]]
  then
    set -- $(grep gateway /etc/sysconfig/interfaces)
    EthGateway=$2
  else
    EthGateway=$EthIP
  fi


  echo ; echo ; echo ; echo
  echo $'\a############################################################'
  echo "Current EtherNet Port Configuration:"
  echo   "  MAC address : $MacAddress"

  if [[ $EthMode = "static" ]]
  then
    echo "  Mode        : static IP"
  else
    echo "  Mode        : dhcp"
  fi

  echo   "  IP          : $EthIP"
  echo   "  Netmask     : $EthNetmask"
  echo   "  Gateway     : $EthGateway"
  echo

  if ! ReadYesNo  "Do you want to change Ethernet Configuration?" "no" 10
  then
    echo
    exit
  fi

  while :
  do
    echo -n "New Mode (static|dhcp): "
    read NewMode
    case "$NewMode" in
    s*|S*) NewMode=static ; break ;;
    d*|D*) NewMode=dhcp ; break ;;
    esac
  done


  if [[ $NewMode = 'static' ]]
  then
    GetIP NewEthIP "New static IP Address" $EthIP
    GetIP NewEthGateway "New Gateway Address" $EthGateway
    GetIP NewEthNetmask "New Netmask" $EthNetmask
  fi

  echo
  echo "New Configuration"
  if [[ $NewMode = 'static' ]]
  then
    echo "  Mode    : static IP"
    echo "  IP      : $NewEthIP"
    echo "  Gateway : $NewEthGateway"
    echo "  Netmask : $NewEthNetmask"
  else
    echo "  Mode    : dhcp"
  fi
  echo


  if ReadYesNo  "Accept New Configuration?" "yes"
  then
    NewConfig
  else
    echo "Changes Rejected"
  fi

done

