#!/usr/bin/perl -w
use strict ;
use CGI qw/:standard/;
use lib '..' ;
use Fireball ;
startPage( 'Set Reference Position' ) ;
sub strip
{
$_[0] =~ s/^\s+// ;
$_[0] =~ s/\s+$// ;
return $_[0] ;
}
my $errorMessage = '' ;
my ( $X, $Y, $Z ) ;
my ( $latDeg, $latMin, $latSec, $NS,
$lonDeg, $lonMin, $lonSec, $EW,
$height
) ;
my $fixedPointPattern = '^[+-]?(\d*\.\d+|\d+\.\d*|\d+)$' ;
my $posFixedPointPattern = '^(\d*\.\d+|\d+\.\d*|\d+)$' ;
my $lat ;
my $lon ;
open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' );
my (undef, undef, undef,
$currentLat, $currentLon, $currentHeight)
= split( ' ', <SYSINFO>) ;
close( SYSINFO ) ;
my $diff ;
if (param('Mode') eq 'lla')
{
$latDeg = strip( param( 'latDeg' ) ) ;
$latMin = strip( param( 'latMin' ) ) ;
$latSec = strip( param( 'latSec' ) ) ;
$NS     = param( 'NS' ) ;
$lonDeg = strip( param( 'lonDeg' ) ) ;
$lonMin = strip( param( 'lonMin' ) ) ;
$lonSec = strip( param( 'lonSec' ) ) ;
$EW     = param( 'EW' ) ;
$height = strip( param( 'Height' ) ) ;
if ( $latDeg !~ m/^\d+$/
or $latMin !~ m/^\d+$/
or $latSec !~ m/$posFixedPointPattern/
or $lonDeg !~ m/^\d+$/
or $lonMin !~ m/^\d+$/
or $lonSec !~ m/$posFixedPointPattern/
or $height !~ m/$fixedPointPattern/
or $latMin >= 60
or $latSec >= 60.0
or ($latDeg+$latMin/60.0+$latSec/3600.0) > 90.0
or $lonMin >= 60
or $lonSec >= 60.0
or ($lonDeg+$lonMin/60.0+$lonSec/3600.0) > 180.0
)
{
$errorMessage = "Lat/Lon/Alt field(s) are not all valid." ;
goto PARAMDONE ;
}
$lat = $latDeg + $latMin/60 + $latSec/3600 ;
$lat = -$lat if ($NS eq 'S') ;
$lon = $lonDeg + $lonMin/60 + $lonSec/3600 ;
$lon = -$lon if ($EW eq 'W') ;
my $heightDiff = ($currentHeight - $height) ;
my $latDiff = ($currentLat - $lat ) * 60.0 * 1852.0 ;
my $lonDiff = ($currentLon - $lon )
* cos( $lat / 360.0 * 2.0 * 3.1415926535 )
* 60.0 * 1852.0 ;
$diff = sqrt( $heightDiff * $heightDiff
+ $latDiff * $latDiff
+ $lonDiff * $lonDiff
) ;
}
else
{
$X = strip( param( 'X' ) ) ;
$Y = strip( param( 'Y' ) ) ;
$Z = strip( param( 'Z' ) ) ;
if ( $X !~ m/$fixedPointPattern/
|| $Y !~ m/$fixedPointPattern/
|| $Z !~ m/$fixedPointPattern/
)
{
$errorMessage = "XYZ values must all be numeric." ;
goto PARAMDONE ;
}
my ($currentX, $currentY, $currentZ )
= split( ' ',
qx( /usr/local/fireball/ecefConvert lla $currentLat $currentLon $currentHeight )
) ;
my $deltaX = $currentX - $X ;
my $deltaY = $currentY - $Y ;
my $deltaZ = $currentZ - $Z ;
$diff = sqrt( $deltaX * $deltaX
+ $deltaY * $deltaY
+ $deltaZ * $deltaZ
) ;
}
if ($diff > 100)
{
$diff = int( $diff ) ;
print <<EOF ;
<script>
alert( 'New Reference Position is quite different from '
+'Current GPS position. ($diff m)'
+'\\nYou might want to verify it.'
) ;
</script>
EOF
}
my %refStationData ;
if (param('Mode') eq 'lla')
{
$refStationData{'lat'} = $lat ;
$refStationData{'lon'} = $lon ;
$refStationData{'height'} = $height ;
}
else
{
$refStationData{'x'} = $X ;
$refStationData{'y'} = $Y ;
$refStationData{'z'} = $Z ;
}
my $id = param('rtcmID') ;
if ( $id !~ /^ *\d+ *$/
or $id > 1023
)
{
$errorMessage = "Invalid RTCM Station ID" ;
goto PARAMDONE ;
}
$refStationData{'rtcmid'} = $id ;
$id = param('cmrID') ;
if ( $id !~ /^ *\d+ *$/
or $id > 31
)
{
$errorMessage = "Invalid CMR Station ID" ;
goto PARAMDONE ;
}
$refStationData{'cmrid' } = $id ;
$refStationData{'binexid' } = param('binexID') ;
$refStationData{'cmrStationName' } = param('cmrStationName') ;
$refStationData{'rtcmdesc' } = param('rtcmDesc') ;
updateRefStationData( \%refStationData,
(param('Mode') ne 'lla')
) ;
print h2( 'New Reference Station parameters accepted' ) ;
PARAMDONE:
if ( $errorMessage )
{
print h3( $errorMessage ) ;
my $returnURL = '/perl-scripts/refpos.cgi' ;
my $startC = '?' ;
for my $p (param())
{
$returnURL .= $startC ;
$returnURL .= $p.'='.CGI::escape( param($p) ) ;
$startC = '&' ;
}
$returnURL .= '&msg='.CGI::escape($errorMessage) ;
print <<EOF ;
<Script>
alert( "$errorMessage" ) ;
parent.loadAfterDelay( "$returnURL", 1000) ;
</Script>
EOF
}
else
{
print <<EOF;
<Script>
parent.loadAfterDelay( "portSummary.cgi", 1500) ;
</Script>
EOF
}
print end_html() ;
