#!/usr/bin/perl -w
use strict ;
use CGI qw/:standard/;
use Fireball ;
titleWithHelp( 'Receiver Status - Position',
'/positionInfo.html'
) ;
sub printll
{
my $label = $_[0] ;
my $ll = $_[1] ;
my $negative = ($ll < 0.0) ;
my $units ;
if ($negative) {$ll = -$ll} ;
my $degrees = int($ll) ;
$ll = ($ll - $degrees) * 60.0 ;
my $minutes = int($ll) ;
my $seconds = ($ll - $minutes) * 60.0 ;
if ($label =~ "Lat")
{
$units = ($negative ? "S" : "N" ) ;
}
elsif ($label =~ "Lon")
{
$units = ($negative ? "W" : "E" ) ;
}
print sprintf( "<b>%s</b> %3d&deg;%02d'%07.4f\" %s\n",
$label, $degrees, $minutes, $seconds,$units ) ;
}
open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' ) ;
my ($latitude, $longitude, $altitude ) = (0,0,0) ;
my @svsUsed = "None" ;
my ($pdop, $hdop, $vdop, $tdop) = (0,0,0,0) ;
my ($velE, $velN, $velUp) = (0,0,0) ;
my ($clockOffset, $clockDrift) = (0,0) ;
my ($posType, $posCorrections) = ("old", "Autonomous") ;
my $warning = '' ;
my $line = <SYSINFO> ;
chomp $line ;
if ($line !~ /^OK/)
{
my ($timetag, $position) = split(':', $line) ;
($latitude, $longitude, $altitude) = split(' ',$position) ;
@svsUsed = split( ' ', <SYSINFO> ) ;
($pdop, $hdop, $vdop, $tdop) = split( ' ', <SYSINFO> ) ;
($velE, $velN, $velUp) = split( ' ', <SYSINFO> ) ;
($clockOffset, $clockDrift) = split( ' ', <SYSINFO> ) ;
($posType, $posCorrections) = split( ' ', <SYSINFO> ) ;
if ($posType eq 'old' )
{
print h3( 'This is old data.  There is no current position fix.' ) ;
$posType = '<b><font color=red>OLD</font></b>' ;
@svsUsed = () ;
}
$posCorrections = 'Autonomous' if ($posCorrections eq 'autonomous') ;
$posCorrections =~ s/_/-/ ;
}
close(SYSINFO) ;
print '<Table width="100%" border=0>
<tr><td valign="top" width="50%">' ;
print "<b>Position:</b>" ;
print "<pre>" ;
printll( "Lat:", $latitude ) ;
printll( "Lon:", $longitude ) ;
printf( "<b>Hgt:</b> %.3f meters\n", $altitude ) ;
print   "<b>    </b> WGS84, $posType, $posCorrections" ;
print "</pre>" ;
print '<p><b>Velocity:</b>
<pre>' ;
printf( "<b>  North:</b> %6.2f m/sec\n", $velN ) ;
printf( "<b>   East:</b> %6.2f m/sec\n", $velE ) ;
printf( "<b>     Up:</b> %6.2f m/sec\n", $velUp ) ;
print "\n";
my $speed = sqrt($velN*$velN + $velE*$velE) ;
printf( "<b>  Horiz:</b> %6.2f m/sec\n", $speed ) ;
printf( "<b>        </b> %6.2f mph\n",
$speed * 2.22369363 # Converts m/s to miles/hour
) ;
printf( "<b>        </b> %6.2f km/h\n",
$speed * 3.6 # Converts m/s to km/hour
) ;
my $heading = atan2( $velE, $velN ) / 3.1415926 * 180.0 ;
$heading += 360.0 if ($heading < 0 ) ;
printf( "<b>Heading:</b> %5.0f&deg; True North\n", $heading ) ;
print '</pre>' ;
print '
</td>
<td valign="top" width="50%">
' ;
print '<b>Satellites used:</b><pre>';
print join( ' ', sort( {$a <=> $b} @svsUsed) ) ;
print '
</pre>' ;
print '<b>Receiver Clock:</b><br />' ;
my $clockSteering = qx( /usr/local/fireball/sysinfo -p clock_steering );
chomp $clockSteering ;
$clockSteering =~ s/put clock_steering // ;
if ( $clockSteering eq 'yes' )
{
$clockOffset = 0.0 ;
$clockDrift  = 0.0 ;
}
print '<code>' ;
printf( "<br /><b>Offset:</b> %10.6f msec\n",
$clockOffset/2.99792458e5 ) ;
printf( "<br /><b>&nbsp;Drift:</b> %10.3f ppm\n", $clockDrift/2.99792458e2 ) ;
print '</code>' ;
print "<br />Clock steering is $clockSteering." ;
print '<br /><br /><br /><b>Dilutions of Precision:</b> <pre>' ;
print( "\n<b>PDOP:</b>  ", $pdop ) ;
print( "\n<b>HDOP:</b>  ", $hdop ) ;
print( "\n<b>VDOP:</b>  ", $vdop ) ;
print( "\n<b>TDOP:</b>  ", $tdop ) ;
print '
</pre>
</td>
</tr>
</table>
' ;
print hr() ;
print timestamp() ;
my $delay = 1700 - (time() % 2) * 1000 ;
print <<EOF ;
<script>
parent.loadAfterDelay( window.location.href, $delay) ;
</script>
EOF
print end_html() ;
